@@ -83,6 +83,14 @@ pub struct ToolUv {
         )
     )]
     pub dev_dependencies: Option<Vec<pep508_rs::Requirement<VerbatimParsedUrl>>>,
+    #[cfg_attr(
+        feature = "schemars",
+        schemars(
+            with = "Option<Vec<String>>",
+            description = "PEP 508 style requirements, e.g. `flask==3.0.0`, or `black @ https://...`."
+        )
+    )]
+    pub override_dependencies: Option<Vec<pep508_rs::Requirement<VerbatimParsedUrl>>>,
 }
 
 #[derive(Serialize, Deserialize, Default, Debug, Clone, PartialEq, Eq)]
@@ -186,14 +194,19 @@ pub enum Source {
 
 #[derive(Error, Debug)]
 pub enum SourceError {
-    #[error("Cannot resolve git reference `{0}`.")]
+    #[error("Cannot resolve git reference `{0}`")]
     UnresolvedReference(String),
-    #[error("Workspace dependency must be a local path.")]
-    InvalidWorkspaceRequirement,
+    #[error("Workspace dependency `{0}` must refer to local directory, not a Git repository")]
+    WorkspacePackageGit(String),
+    #[error("Workspace dependency `{0}` must refer to local directory, not a URL")]
+    WorkspacePackageUrl(String),
+    #[error("Workspace dependency `{0}` must refer to local directory, not a file")]
+    WorkspacePackageFile(String),
 }
 
 impl Source {
     pub fn from_requirement(
+        name: &PackageName,
         source: RequirementSource,
         workspace: bool,
         editable: Option<bool>,
@@ -202,15 +215,23 @@ impl Source {
         branch: Option<String>,
     ) -> Result<Option<Source>, SourceError> {
         if workspace {
-            match source {
-                RequirementSource::Registry { .. } | RequirementSource::Directory { .. } => {}
-                _ => return Err(SourceError::InvalidWorkspaceRequirement),
-            }
-
-            return Ok(Some(Source::Workspace {
-                editable,
-                workspace: true,
-            }));
+            return match source {
+                RequirementSource::Registry { .. } | RequirementSource::Directory { .. } => {
+                    Ok(Some(Source::Workspace {
+                        editable,
+                        workspace: true,
+                    }))
+                }
+                RequirementSource::Url { .. } => {
+                    Err(SourceError::WorkspacePackageUrl(name.to_string()))
+                }
+                RequirementSource::Git { .. } => {
+                    Err(SourceError::WorkspacePackageGit(name.to_string()))
+                }
+                RequirementSource::Path { .. } => {
+                    Err(SourceError::WorkspacePackageFile(name.to_string()))
+                }
+            };
         }
 
         let source = match source {
