@@ -15,21 +15,19 @@ use distribution_types::{
     DistributionMetadata, IndexLocations, InstalledMetadata, LocalDist, Name, Resolution,
 };
 use install_wheel_rs::linker::LinkMode;
-use pep440_rs::VersionSpecifiers;
 use pep508_rs::MarkerEnvironment;
 use platform_tags::Tags;
 use pypi_types::Requirement;
 use uv_cache::Cache;
 use uv_client::{BaseClientBuilder, RegistryClient};
 use uv_configuration::{
-    Concurrency, Constraints, ExtrasSpecification, NoBinary, Overrides, PreviewMode, Reinstall,
+    BuildOptions, Concurrency, Constraints, ExtrasSpecification, Overrides, PreviewMode, Reinstall,
     Upgrade,
 };
 use uv_dispatch::BuildDispatch;
 use uv_distribution::DistributionDatabase;
 use uv_fs::Simplified;
-use uv_installer::{Downloader, Plan, Planner, SitePackages};
-use uv_interpreter::{Interpreter, PythonEnvironment};
+use uv_installer::{Plan, Planner, Preparer, SitePackages};
 use uv_normalize::{GroupName, PackageName};
 use uv_requirements::{
     LookaheadResolver, NamedRequirementsResolver, RequirementsSource, RequirementsSpecification,
@@ -37,12 +35,13 @@ use uv_requirements::{
 };
 use uv_resolver::{
     DependencyMode, Exclusions, FlatIndex, InMemoryIndex, Manifest, Options, Preference,
-    PythonRequirement, ResolutionGraph, Resolver,
+    Preferences, PythonRequirement, ResolutionGraph, Resolver,
 };
+use uv_toolchain::PythonEnvironment;
 use uv_types::{HashStrategy, InFlight, InstalledPackagesProvider};
 use uv_warnings::warn_user;
 
-use crate::commands::reporters::{DownloadReporter, InstallReporter, ResolverReporter};
+use crate::commands::reporters::{InstallReporter, PrepareReporter, ResolverReporter};
 use crate::commands::{compile_bytecode, elapsed, ChangeEvent, ChangeEventKind, DryRunEvent};
 use crate::printer::Printer;
 
@@ -88,10 +87,9 @@ pub(crate) async fn resolve<InstalledPackages: InstalledPackagesProvider>(
     hasher: &HashStrategy,
     reinstall: &Reinstall,
     upgrade: &Upgrade,
-    interpreter: &Interpreter,
-    tags: &Tags,
+    tags: Option<&Tags>,
     markers: Option<&MarkerEnvironment>,
-    requires_python: Option<&VersionSpecifiers>,
+    python_requirement: PythonRequirement,
     client: &RegistryClient,
     flat_index: &FlatIndex,
     index: &InMemoryIndex,
@@ -185,11 +183,7 @@ pub(crate) async fn resolve<InstalledPackages: InstalledPackagesProvider>(
     // Collect constraints and overrides.
     let constraints = Constraints::from_requirements(constraints);
     let overrides = Overrides::from_requirements(overrides);
-    let python_requirement = if let Some(requires_python) = requires_python {
-        PythonRequirement::from_requires_python(interpreter, requires_python)
-    } else {
-        PythonRequirement::from_interpreter(interpreter)
-    };
+    let preferences = Preferences::from_iter(preferences, markers);
 
     // Determine any lookahead requirements.
     let lookaheads = match options.dependency_mode {
@@ -198,6 +192,7 @@ pub(crate) async fn resolve<InstalledPackages: InstalledPackagesProvider>(
                 &requirements,
                 &constraints,
                 &overrides,
+                &dev,
                 hasher,
                 index,
                 DistributionDatabase::new(client, build_dispatch, concurrency.downloads, preview),
@@ -289,7 +284,7 @@ pub(crate) async fn install(
     site_packages: SitePackages,
     modifications: Modifications,
     reinstall: &Reinstall,
-    no_binary: &NoBinary,
+    build_options: &BuildOptions,
     link_mode: LinkMode,
     compile: bool,
     index_urls: &IndexLocations,
@@ -316,7 +311,7 @@ pub(crate) async fn install(
         .build(
             site_packages,
             reinstall,
-            no_binary,
+            build_options,
             hasher,
             index_urls,
             cache,
@@ -375,25 +370,25 @@ pub(crate) async fn install(
     } else {
         let start = std::time::Instant::now();
 
-        let downloader = Downloader::new(
+        let preparer = Preparer::new(
             cache,
             tags,
             hasher,
             DistributionDatabase::new(client, build_dispatch, concurrency.downloads, preview),
         )
-        .with_reporter(DownloadReporter::from(printer).with_length(remote.len() as u64));
+        .with_reporter(PrepareReporter::from(printer).with_length(remote.len() as u64));
 
-        let wheels = downloader
-            .download(remote.clone(), in_flight)
+        let wheels = preparer
+            .prepare(remote.clone(), in_flight)
             .await
-            .context("Failed to download distributions")?;
+            .context("Failed to prepare distributions")?;
 
         let s = if wheels.len() == 1 { "" } else { "s" };
         writeln!(
             printer.stderr(),
             "{}",
             format!(
-                "Downloaded {} in {}",
+                "Prepared {} in {}",
                 format!("{} package{}", wheels.len(), s).bold(),
                 elapsed(start.elapsed())
             )
@@ -694,7 +689,7 @@ pub(crate) fn diagnose_environment(
     venv: &PythonEnvironment,
     printer: Printer,
 ) -> Result<(), Error> {
-    let site_packages = SitePackages::from_executable(venv)?;
+    let site_packages = SitePackages::from_environment(venv)?;
     for diagnostic in site_packages.diagnostics()? {
         // Only surface diagnostics that are "relevant" to the current resolution.
         if resolution
@@ -738,4 +733,7 @@ pub(crate) enum Error {
 
     #[error(transparent)]
     Anyhow(#[from] anyhow::Error),
+
+    #[error(transparent)]
+    PubGrubSpecifier(#[from] uv_resolver::PubGrubSpecifierError),
 }
